%Created:       12.07.2012 by Peter Kandolf
%Last edit: 	12.07.2012 by Peter Kandolf
%Version:       0
%Author:        Peter Kandolf
%Remarks:
%
%Interface:
% [A,H,v] = AdvectionDiffusion2D(DIMENSION, PECLET)
%
% Computes an advection diffusion operator of size DIMENSIONxDIMENSION with
% specified PECLET number. It also returns a range of sugested step sizes
% and an appropriate initial vector.
%
%See also PHILEJA
%--------------------------------------------------------------------------
%Changes:
%   12.07.12 (PK):  changes in version 0
%                   file created
function [A,k,v]=AdvectionDiffusion2D(dimension, Peclet)
N = dimension;
h = 1/(N+1); % interval [0,1], homogeneous Dirichlet boundary conditions
epsilon = 1;
c = 2*epsilon*Peclet/h;
A = epsilon * toeplitz(sparse([1,1],[1,2],[-2,1]/h^2,1,N));

B = c * toeplitz(sparse(1,2,-1/(2*h),1,N),sparse(1,2,1/(2*h),1,N));
A = (kron(A+B,speye(N))+kron(speye(N),A+B));
x = linspace(0,1,N+2);
x = x(2:N+1);
[X,Y] = meshgrid(x);
v = 256*X.^2.*(1-X).^2.*Y.^2.*(1-Y).^2; %normalized
v = v(:); % appropriate initial vector
k=[1e-4,1e-2];