%Created:       12.07.2012 by Peter Kandolf
%Last edit: 	12.07.2012 by Peter Kandolf
%Version:       0
%Author:        Peter Kandolf
%Remarks:
%
%Interface:
% [A,H,v] = Schroedinger3d(DIMENSION, dx, factor)
%
% Computes an Schrödinger operator of size DIMENSIONxDIMENSIONxDIMENSION with
% respect to the given interval. Factor specifies the leading term for the
% harmonic potential.  It also returns a range of sugested step sizes
% and an appropriate initial vector.
%
%See also PHILEJA
%--------------------------------------------------------------------------
%Changes:
%   12.07.12 (PK):  changes in version 0
%                   file created
function [A,k,v]=Schroedinger3d(dimension, interval,factor)
N = dimension;
a=interval(1);
b=interval(2);
h=(b-a)/(N+1); %homogenious dirichlet boundary conditions

B = toeplitz(sparse([1,1],[1,2],[2,-1]/2,1,N)) / h^2; % -1/2*laplace
I=speye(N);
A = kron(I,kron(I,B))+kron(I,kron(B,I))+kron(B,kron(I,I));
x = linspace(a,b,N+2)';
x = x(2:N+1);
[X,Y,Z] = ndgrid(x);
harmonicPotential=factor*(X(:).^2+Y(:).^2+Z(:).^2)/2; % 1/2*V
k=[5e-3,5e-1];
A = -1i*(A+spdiags(harmonicPotential,0,N^3,N^3));
v=(2/(b-a))^12*(a-X).^2.*(b-X).^2.*(a-Y).^2.*(b-Y).^2.*(a-Z).^2.*(b-Z).^2;%normalized
v=v(:);
