function [L,k,v]=spaceapproximation(N,M,a1,b1,a2,b2,Dx,Dy,mu)
%Copyright  Antoine Tambue
%ADR = advection diffusion reaction 
% Finite Volume with two point flux approximation in rectangular grid
% [a1,b1]X[a2 b2], Dirichlet  boundary condition at  x=a1, and  Neuman
% boundary condition elsewhere, see (Tambue et al. 2010  Journal of
  % computational physics) 
  %The code is expanded to allow beginners to understand the space discretization of finite volume and 
%can be optimized easily to save CPU time.
  %This function solve ADR with simple semi implicit method, exponential integrators can be used  just following our few  comments
 % Following the code we first build the ODes  U'=LU+N1  as in equation (6)  see (Tambue et al. 2010  Journal of
  % computational physics),  N1 here represent the vector function N in (6)
  %The output is  the matrix L. 
doplot=false;
%ep=1e-3;
%Dx=1*ep; Dy=0.1*ep;  % for diffusion tensor
%N=100;
%M=100;
%T1=0.0; 
%a1=0; b1=3; a2=0;b2=2; 
%Cin=1;
h1=(b1-a1)/N; 
h2=(b2-a2)/M;

%mu=1; % dymanical viscosity 

 %[X,Y]=meshgrid([a1,a1+0.5*h1:h1:b1-0.5*h2,b1],[a2,a2+0.5*h2:h2:b2-0.5*h2,b2]);
 %[X,Y]=meshgrid([a1+0.5*h1:h1:b1-0.5*h2],[a2+0.5*h2:h2:b2-0.5*h2]);
 
[vx,vx1,vy,vy1]=velocityhh1(N,M,a1,b1,a2,b2,mu); % velocity per grid point

T=1; % final time 

vx=vx'; vx1=vx1';vy1=vy1';vy=vy';

NM=N*M;
be=h2/h1;
Dx=be*Dx;
Dy=Dy/be;

l1=(2*Dx+2*Dy)*ones(M,1); 
l1(1)= l1(1)-Dy;
l1(M)=l1(1);
l=repmat(l1,[N,1]);
l(1:M)=l(1:M)+Dx*ones(M,1);
l((NM-M+1):NM)=l((NM-M+1):NM)-Dx*ones(M,1);
A=spdiags([l],0,NM,NM);

sg=upwind(vy(1,1));
sg1=upwind(vx(1,1));
A(1,1)=A(1,1)+sg1*vx(1,1)*h2+upwind(vx1(1,1))*vx1(1,1)*h2+sg*vy(1,1)*h1;
A(1,2)=-Dy+(1-sg)*vy(1,1)*h1;
A(1,1+M)=-Dx+(1-sg1)*vx(1,1)*h2;
sg=upwind(vx(1,M));
sg1=upwind(vy1(1,M));
A(M,M)=A(M,M)+sg*vx(1,M)*h2+upwind(vx1(1,M))*vx1(1,M)*h2+sg1*vy1(1,M)*h1;
A(M,M-1)=-Dy+(1-sg1)*vy1(1,M)*h1;
A(M,2*M)=-Dx+(1-sg)*vx(1,M)*h2;
for i=2:M-1
    sg=upwind(vx(1,i));
    sg1=upwind(vy(1,i));
    sg2=upwind(vy1(1,i));
   A(i,i)=A(i,i)+ sg*vx(1,i)*h2+ upwind(vx1(1,i))*vx1(1,i)*h2+sg1*vy(1,i)*h1+sg2*vy1(1,i)*h1;
   A(i,i-1)=-Dy+(1-sg2)*vy1(1,i)*h1;
   A(i,i+1)=-Dy+(1-sg1)*vy(1,i)*h1;
   A(i,i+M)=-Dx+(1-sg)*vx(1,i)*h2;
end
sg=upwind(vx1(N,1));
sg1=upwind(vy(N,1));
A(NM-M+1,NM-M+1)= A(NM-M+1,NM-M+1)+upwind(vy1(N,1))*vy1(N,1)*h1+sg*vx1(N,1)*h2+sg1*vy(N,1)*h1+h2*vx(N,1);
A(NM-M+1,NM-2*M+1)=-Dx+(1-sg)*vx1(N,1)*h2;
A(NM-M+1,NM-M+2)=-Dy+(1-sg1)*vy(N,1)*h1;   
sg=upwind(vx1(N,M));
sg1=upwind(vy1(N,M));
A(NM,NM)=A(NM,NM)+upwind(vy(N,M))*vy(N,M)*h1+sg*vx1(N,M)*h2+sg*vy1(N,M)*h1+h2*vx(N,M);
A(NM,NM-1)=-Dy+(1-sg1)*vy1(N,M)*h1;
A(NM,NM-M)=-Dx+(1-sg)*vx1(N,M)*h2;
for i=(NM-M+2):NM-1
    j=mod(i,M);
    sg=upwind(vx1(N,j));
    sg1=upwind(vy(N,j));
    sg2=upwind(vy1(N,j));
    A(i,i)=A(i,i)+ sg1*vy(N,j)*h1+sg*vx1(N,j)*h2+sg2*vy1(N,j)*h1+h2*vx(N,j);
    A(i,i-1)=-Dy+(1-sg2)*vy1(N,j)*h1;
    A(i,i+1)=-Dy+(1-sg1)*vy(N,j)*h1;
    A(i,i-M)=-Dx+(1-sg)*vx1(N,j)*h2;
end
for i=M+1:(NM-M)
    j=mod(i,M);
    if j==1
        ii=1+(i-1)/M;
       sg=upwind(vx(ii,1));
       sg1=upwind(vy(ii,1));
       sg2=upwind(vx1(ii,1));
        A(i,i)=A(i,i)+sg*vx(ii,1)*h2+sg1*vy(ii,1)*h1+sg2*vx1(ii,1)*h2;
        A(i,i+1)=-Dy+(1-sg1)*vy(ii,1)*h1;
        A(i,i+M)=-Dx+(1-sg)*vx(ii,1)*h2;
        A(i,i-M)=-Dx+(1-sg2)*vx1(ii,1)*h2;
    elseif j==0
        ii=(i)/M;
        sg=upwind(vx(ii,M));
        sg1=upwind(vy1(ii,M));
        sg2=upwind(vx1(ii,M));
        A(i,i)=A(i,i)+sg*vx(ii,M)*h2+sg1*vy1(ii,M)*h1+sg2*vx1(ii,M)*h2;
        A(i,i-1)=-Dy+(1-sg1)*vy1(ii,M)*h1;
        A(i,i+M)=-Dx+(1-sg)*vx(ii,M)*h2;
        A(i,i-M)=-Dx+(1-sg2)*vx1(ii,M)*h2;

    else
        ii=1+(i-j)/M;
        sg=upwind(vx(ii,j));
        sg1=upwind(vy(ii,j));
        sg2=upwind(vy1(ii,j));
        sg3=upwind(vx1(ii,j));
        A(i,i)= A(i,i)+sg*vx(ii,j)*h2+sg1*vy(ii,j)*h1+sg2*vy1(ii,j)*h1+sg3*vx1(ii,j)*h2;
        A(i,i-M)= -Dx+(1-sg3)*vx1(ii,j)*h2;
        A(i,i-1)= -Dy+(1-sg2)*vy1(ii,j)*h1;
        A(i,i+1)=-Dy+(1-sg1)*vy(ii,j)*h1;
        A(i,i+M)=-Dx+(1-sg)*vx(ii,j)*h2;
    end
end

MM=10;
tstep=T/MM;
U=zeros(NM,1);  % Initial solution
L=-(1/(h1*h2))*A;  % This  is now  the matrix L in  equation (6) (Tambue
		   % et al. 2010 )
k=[1e-3,1e-1];
x = linspace(a1,b1,N);
y = linspace(a2,b2,M);
[X,Y] = meshgrid(x,y);
v = (2/(b1-a1)*2/(b2-a2))^4*(a1-X).^2.*(b1-X).^2.*(a2-Y).^2.*(b2-Y).^2;%normalized
v = v(:); % appropriate initial vector
rng('default');
return

J=(speye(N*M)+tstep*(-L)); % Backward Euler: it means y'(t) = L y(t)
    [l,u]=lu(J);
for j=1:MM
    
   
   F=-U./(1+U);  % This is just an example of the reaction function, not yet N in  equation (6)( Tambue et al. 2010 )
   
   
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    % This the contribution of the boundary condition and can be evaluated
    % outside  this loop if the boundary condition is not time depend as in our example.
    G = bv(N,M,Dx,h2,vx1);
    
       g = zeros(NM,1);   % required vector of BVs, can be write in sparse form for memory saving 
        % left edge
        g(1:M) = G(2:M+1,1);
        % right edge
        g(NM-M+1:NM) = G(2:M+1,N+2);
        % bottom 
        g(1:M:NM) = g(1:M:NM) + G(1,2:N+1)';
            % top
        g(M:M:NM) = g(M:M:NM) +G(M+2,2:N+1)';
        F1=(1/(h1*h2))*g;  % This is the contribution of the boundary condition in  the vector N (equation (6) Tambue et al. 2010)
        N1=F1+F; %  This  is now  the whole vector N, in  equation (6) 
       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       
       % Here we are solving the equation ODE  U'=LU+N1 with  semi implicit
       % method. This is the place to used any  time stepping schemes, in particular  exponential integrators
       % schemes
       
        U=u\(l\((tstep)*N1+U));   %  simple semi implicit numerical solution
       
       
end
if doplot
figure(1)
 U= reshape(U,M,N);
 surf(X,Y,U);
shading interp
xlabel('x')
ylabel('y')
colorbar
end
 %mesh(U)

function G = bv(N,M,Dx,h2,vx1)
%G is  boundary condition function, which can be value for dirichlet or
%fluxes for Neuman. We treat G as an (M+2)x(M+2) grid function on the entire grid
% including boundaries. Only the boundary values are defined
G=spdiags([0],-M-1,M+2,N+2);
vx1=vx1';
%at x=a1 
G(2:M+1,1)=(2*Dx-(1-upwind(vx1(1:end,1)))*h2.*vx1(1:end,1)).*ones(M,1);   
% ones(M,1) is  here  as we took  for value at Dirichlet equal to 1, any function can be used
 
%%%%%%%%% Here we show  how to deal with no zeros value if needed,
% at x=b1
% G(2:M+1,N+2) =zeros(M,1); % just put the corresponding value and not zeros(M,1)
%  %at y=a2
% G(1,2:N+1) =zeros(1,N);  % just put the corresponding value and not zeros(1,N)
% % y=b2
% G(M+2,2:N+1)=zeros(1,N); % just put the corresponding value and not not zeros(1,N)

function r=upwind(a)
 r=0.5*(sign(a)+1);

function [vx,vx1,vy,vy1]=velocityhh1(N,M,a1,b1,a2,b2,mu)
doplot=false;
%Use a general finite volume method (Eymard et all. 1997 or 2000)  in
%cartesien grid  on rectangular domain [a1,b1]X[a2 b2]
  % Boundry condition are specified in (Tambue et al. 2010  Journal of computational physics) section 4
%The code is expanded to allow beginners to understand the space discretization of finite volume and 
%can be optimized easily to save CPU time
% The output is vx,vx1,vy,vy where vx is the velocity in positive x
% direction and vx1 the velocity in negative x direction in each  rectangular grid, same for vy, vy1.
% mu here is the dynamical viscosity
% Note that pressures used are reference presures (ie. p=0, physically is non sense, but here just means  the low pressure,  
%it can be fixed as  1 bar for example  and p=1  can be fixed as 10 bar for example) 

h1=(b1-a1)/(N); h2=(b2-a2)/M;
be=h2/h1;
NM=N*M;

[X,Y]=meshgrid(a1+0.5*h1:h1:b1-0.5*h1,a2+0.5*h2:h2:b2-0.5*h2);


% Pemearbility in fracture grid in Fig 4 (Tambue et al. 2010  Journal of computational physics)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   P=ones(N,M);
%    Mi=ceil(M/4);
%    Ni=ceil(N/5);
%     P(Ni:2*Ni-1,3*(Mi))=100*ones(Ni,1);
%    P(2*Ni:3*Ni-1,2*(Mi))=100*ones(Ni,1);
%    P(3*Ni:4*Ni-1,1*(Mi))=100*ones(Ni,1);
   
 % Ramdon Pemearbility as Fig 6 (Tambue et al. 2010  Journal of compuational physics) 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 P=permea3(N,M,b1,b2);

T=spdiags([0],-NM+1,NM,NM);
p1=P(1,1);                   %permea(0.5*h,0.5*h,N);
p2=P(1,2)  ;                %p2=permea(0.5*h,1.5*h,N);
p3= P(2,1); 
T(1,2)=-2*(p2*p1)/(be*(p2+p1));
T(1,M+1)=-2*be*(p3*p1)/(p3+p1);
T(1,1)=2*be*p1-T(1,2)-T(1,M+1);
p1=P(1,M);  %p1=permea(0.5*h,(N-0.5)*h,N);
p2 =P(1,M-1);%p2=permea(0.5*h,(N-1.5)*h,N);
p3=P(2,M);  %  p3=permea(1.5*h,(N-0.5)*h,N);

T(M,M-1)=-2*(p2*p1)/(be*(p2+p1));
T(M,2*M)=-2*be*(p3*p1)/(p3+p1);
T(M,M)=2*be*p1-T(M,M-1)-T(M,2*M);

p1=P(N,1); %p1=permea((N-0.5)*h,(0.5)*h,N);
p2=P(N-1,1);  %permea((N-1.5)*h,(0.5)*h,N);
p3=P(N,2); %p3=permea((N-0.5)*h,(1.5)*h,N);
T(NM-M+1,NM-2*M+1)=-2*be*(p2*p1)/(p2+p1);
T(NM-M+1,NM-M+2)=-2*(p3*p1)/(be*(p3+p1));
T(NM-M+1,NM-M+1)=2*be*p1-T(NM-M+1,NM-2*M+1)-T(NM-M+1,NM-M+2);
p1=P(N,M);          %p1=permea((N-0.5)*h,(N-0.5)*h,N);
p2=P(N,M-1);          %p2=permea((N-0.5)*h,(N-1.5)*h,N);
p3=P(N-1,M);          %p3=permea((N-1.5)*h,(N-0.5)*h,N);
T(NM,NM-1)=-2*(p2*p1)/(be*(p2+p1));
T(NM,NM-M)=-2*be*(p3*p1)/(p3+p1);
T(NM,NM)=2*be*p1-T(NM,NM-1)-T(NM,NM-M);
for i=[2:(M-1),(NM-M+2):NM-1]
    if  i<M
        p1=P(1,i);    %p1=permea((0.5)*h,(i-0.5)*h,N);
        p2=P(1,i-1);  %p2=permea(0.5*h,(i-1.5)*h,N);
        p3=P(1,i+1);   %p3=permea(0.5*h,(i+0.5)*h,N);
        p4=P(2,i) ;  %p4=permea(1.5*h,(i-0.5)*h,N);
        T(i,i-1)=-2*(p2*p1)/(be*(p2+p1));
        T(i,i+1)=-2*(p3*p1)/(be*(p3+p1));
        T(i,i+M)=-2*be*(p4*p1)/(p4+p1);
        T(i,i)=2*be*p1-T(i,i-1)-T(i,i+1)-T(i,i+M);

    else
        j=mod(i,M);
        ii=N;
        p1=P(N,j);%p1=permea((N-0.5)*h,(j-0.5)*h,N);
        p2=P(N-1,j);    %p2=permea((N-1.5)*h,(j-0.5)*h,N);
        p3=P(N,j-1);  %p3=permea((N-0.5)*h,(j-1.5)*h,N);
        p4=P(N,j+1);               %p4=permea((N-0.5)*h,(j+0.5)*h,N);
        T(i,i-M)=-2*be*(p2*p1)/(p2+p1);
        T(i,i+1)=-2*(p4*p1)/(be*(p4+p1));
        T(i,i-1)=-2*(p3*p1)/(be*(p3+p1));
        T(i,i)=2*be*p1- T(i,i-M)-T(i,i+1)-T(i,i-1);
    end
end
for i=M+1:(NM-M)
    j=mod(i,M);
    if j==1
        ii=1+(i-1)/M;
        p1=P(ii,1); %p1=permea((ii-0.5)*h,0.5*h,N);
        p2=P(ii+1,1); %p2=permea((ii+0.5)*h,0.5*h,N);
        p3=P(ii-1,1);  %p3=permea((ii-1.5)*h,0.5*h,N);
        p4=P(ii,2); %p4=permea((ii-0.5)*h,1.5*h,N);
        T(i,i+1)=-2*(p1*p4)/(be*(p1+p4));
        T(i,i+M)=-2*be*(p2*p1)/(p2+p1);
        T(i,i-M)=-2*be*(p3*p1)/(p3+p1);
        T(i,i)=-T(i,i+M)-T(i,i-M)-T(i,i+1);
    elseif j==0
        ii=(i)/M;
        p1=P(ii,M); % p1=permea((ii-0.5)*h,(N-0.5)*h,N);
        p2=P(ii+1,M); %p2=permea((ii+0.5)*h,(N-0.5)*h,N);
        p3=P(ii-1,M);  %p3=permea((ii-1.5)*h,(N-0.5)*h,N);
        p4=P(ii,M-1);  %p4=permea((ii-0.5)*h,(N-1.5)*h,N);
        T(i,i-1)=-2*(p4*p1)/(be*(p4+p1));
        T(i,i+M)=-2*be*(p2*p1)/(p2+p1);
        T(i,i-M)=-2*be*(p3*p1)/(p3+p1);
        T(i,i)=-T(i,i+M)-T(i,i-M)-T(i,i-1);

    else
        ii=1+(i-j)/M;
       p1=P(ii,j); % p1=permea((ii)*h-0.5*h,j*h-0.5*h,N);
       p2=P(ii+1,j); % p2=permea((ii+1)*h-0.5*h,j*h-0.5*h,N);
       p3=P(ii-1,j);  % p3=permea((ii-1)*h-0.5*h,j*h-0.5*h,N);
       p4=P(ii,j+1);  % p4=permea((ii)*h-0.5*h,(j+1)*h-0.5*h,N);
       p5=P(ii,j-1);  % p5=permea((ii)*h-0.5*h,(j-1)*h-0.5*h,N);
        T(i,i-M)=-2*be*(p3*p1)/(p3+p1);
        T(i,i-1)=-2*(p5*p1)/(be*(p5+p1)) ;
        T(i,i+1)=-2*(p4*p1)/(be*(p4+p1));
        T(i,i+M)=-2*be*(p2*p1)/(p2+p1);
        T(i,i)= -T(i,i-M)-T(i,i-1)- T(i,i+1)-T(i,i+M);
    end
end
l=[];
for i=1:M
    l=[l;2*be*P(1,i)];    %permea(0.5*h,(i-0.5)*h,N)];
end  

F=[l;zeros(M*(N-1),1)];
U=T\F;  %iterative method can be used for large system


A=reshape(U,M,N);  

vx=zeros(M,N);
vy=zeros(M,N);
for i=1:M
    for j=1:N
         if j==N
             vx(i,N)=2*be*P(N,i)*(A(i,N))/h2;  %permea((i-0.5)*h,(N-0.5)*h,N)
         else
           vx(i,j)=(2*be*P(j,i)*P(j+1,i)/(P(j,i)+P(j+1,i)))*(A(i,j)-A(i,j+1))/h2; %permea((i-0.5)*h,(j-0.5)*h,N)
         end
         if i==M
             vy(M,j)=0;
         else
             vy(i,j)=(2*P(j,i)*P(j,i+1)/(be*(P(j,i)+P(j,i+1))))*(A(i,j)-A(i+1,j))/h1; % permea((i-0.5)*h,(j-0.5)*h,N)
         end
    end
end
vx1=zeros(M,N);
vy1=zeros(M,N);
for i=1:M
    for j=1:N
         if j==1
             vx1(i,1)=2*be*P(1,i)*(A(i,1)-1)/h2;  %permea((i-0.5)*h,(N-0.5)*h,N)
         else
           vx1(i,j)=((2*be*P(j,i)*P(j-1,i))/(P(j,i)+P(j-1,i)))*(A(i,j)-A(i,j-1))/h2; %permea((i-0.5)*h,(j-0.5)*h,N)
         end
         if i==1
             vy1(1,j)=0;
         else
             vy1(i,j)=((2*P(j,i)*P(j,i-1))/(be*(P(j,i)+P(j,i-1))))*(A(i,j)-A(i-1,j))/h1; % permea((i-0.5)*h,(j-0.5)*h,N)
         end
    end
end

vx=vx./mu;
vy=vy./mu;
vx1=vx1./mu;
vy1=vy1./mu;

if doplot
% streamline of the velocity field
figure(2)
streamslice(X,Y,vx,vy,5,'cubic')
axis([a1 b1 a2 b2])

div=(vx+vx1)/h1+(vy+vy1)/h2;
% div is in fact the divergence of the velocity per grid in order to see how
% accurate is the numerical solution, we have plot it in figure 3.  Note
% the exact value is zero as we are solving  the div(v)=0.
%The method  used here is efficient and can be some time
% less acurate in higly heterogeneous media. In this  case people used the so called hybrid finite volume method
% where  pressure and flux (or velocity) are found in the same time.

figure(3)
mesh(X,Y,div)
axis([a1,b1 a2 b2])
colorbar

% pressure graph
figure(4)
surf(X,Y,A)
shading interp
xlabel('x')
ylabel('y')
colorbar
 %  This just the norm of the velocity per grid
figure(5)
surf(X,Y,sqrt(vx.^2+vy.^2))
xlabel('x')
ylabel('y')
shading interp
colorbar
end
function k=permea3(N,M,b1,b2)
doplot=false;
% b1=3;
% b2=2;
 %N=30;
 %M=30;
%randn('state',100)    is used in the article
rng(100)    %MATLAB 2013b new notation
h1=b1/N; % step in x direction
h2=b2/M;  % step in y direction
x=0.5*h1:h1:b1-0.5*h1'; % This is just an example of grid points in x direction , it can be change
y= 0.5*h2:h2:b2-0.5*h2';    
k=zeros(M,N);
xi=0.4;   % correlation length is  x direction
xi1=0.2;  % correlation length in y direction

R=randn(M,N); % independent gaussian random variable

[X,Y]=meshgrid(0:N-1,0:M-1);
B=R.*exp(-0.5*pi*((xi*X./b1).^2+(xi1*Y./b2).^2));
k=sqrt(N*M)*idct2(B)/sqrt(b1*b2);
%k=exp(k');
if doplot
figure(6)
[X,Y]=meshgrid(x,y);
surf(x,y,k)
shading interp;
 view(2)
xlabel('x')
 ylabel('y')
 title(['xcorrealation=',  num2str(xi) ,'   ycorrelation=',num2str(xi1)])
end
k=exp(k');

              
